# -*- coding: utf-8 -*-
# Generated by Django 1.11.11 on 2020-01-14 21:55
from __future__ import unicode_literals

from django.contrib.postgres.aggregates import ArrayAgg
from django.db import migrations
from django.db.models import Count


def delete_duplicate_interfaces(apps, schema_editor):
    Node = apps.get_model("maasserver", "Node")
    Interface = apps.get_model("maasserver", "Interface")
    ScriptResult = apps.get_model("metadataserver", "ScriptResult")
    Neighbour = apps.get_model("maasserver", "Neighbour")
    MDNS = apps.get_model("maasserver", "MDNS")

    # This is to remove the default ordering as Meta class for Interface
    # has ordering by created.
    duplicates = (
        Interface.objects.all()
        .values("name", "node_id")
        .annotate(if_ids=ArrayAgg("id"), name_count=Count("id"))
        .exclude(node_id__isnull=True)
        .filter(name_count__gt=1)
        .order_by()
    )

    for duplicate in duplicates:
        node = Node.objects.get(id=duplicate["node_id"])
        if_ids = duplicate["if_ids"]
        nic_keeping_id = if_ids[0]
        for duplicate_nic_id in if_ids[1:]:
            # Reset boot_interface_id for this node.
            if node.boot_interface_id == duplicate_nic_id:
                node.boot_interface_id = nic_keeping_id
                node.save()

            # Update MDNS.
            MDNS.objects.filter(interface_id=duplicate_nic_id).update(
                interface_id=nic_keeping_id
            )

            # Update ScriptResult.
            ScriptResult.objects.filter(interface_id=duplicate_nic_id).update(
                interface_id=nic_keeping_id
            )

            # Update Neighbour.
            Neighbour.objects.filter(interface_id=duplicate_nic_id).update(
                interface_id=nic_keeping_id
            )

            # Delete the duplicate interface.
            Interface.objects.filter(id=duplicate_nic_id).delete()


class Migration(migrations.Migration):

    dependencies = [("maasserver", "0202_event_node_on_delete")]

    operations = [migrations.RunPython(delete_duplicate_interfaces)]
