# Generated by Django 2.2.12 on 2020-08-24 13:43

import django.contrib.postgres.fields
from django.db import migrations, models
import django.db.models.deletion
from django.db.models.expressions import RawSQL
from django.utils import timezone

import maasserver.models.cleansave


def create_virtual_machines(apps, schema_editor):
    Node = apps.get_model("maasserver", "Node")
    VirtualMachine = apps.get_model("maasserver", "VirtualMachine")
    nodes_info = (
        Node.objects.filter(bmc__power_type="lxd")
        .annotate(
            identifier=RawSQL(
                "instance_power_parameters->>'instance_name'", ()
            )
        )
        .values(
            "memory",
            "bmc_id",
            "identifier",
            machine_id=models.F("id"),
            unpinned_cores=models.F("cpu_count"),
        )
    )

    now = timezone.now()
    VirtualMachine.objects.bulk_create(
        VirtualMachine(**info, created=now, updated=now) for info in nodes_info
    )


class Migration(migrations.Migration):

    dependencies = [("maasserver", "0212_notifications_fields")]

    operations = [
        migrations.CreateModel(
            name="VirtualMachine",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("created", models.DateTimeField(editable=False)),
                ("updated", models.DateTimeField(editable=False)),
                ("identifier", models.TextField()),
                (
                    "pinned_cores",
                    django.contrib.postgres.fields.ArrayField(
                        base_field=models.IntegerField(),
                        blank=True,
                        default=list,
                        size=None,
                    ),
                ),
                ("unpinned_cores", models.IntegerField(blank=True, default=0)),
                ("memory", models.IntegerField(default=0)),
                ("hugepages_backed", models.BooleanField(default=False)),
                (
                    "bmc",
                    models.ForeignKey(
                        editable=False,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="maasserver.BMC",
                    ),
                ),
                (
                    "machine",
                    models.ForeignKey(
                        blank=True,
                        default=None,
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="maasserver.Machine",
                    ),
                ),
            ],
            options={"unique_together": {("bmc", "identifier")}},
            bases=(
                maasserver.models.cleansave.CleanSave,
                models.Model,
                object,
            ),
        ),
        migrations.RunPython(create_virtual_machines),
    ]
